@@ -15,15 +15,14 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * This file includes excerpts from the Document Object Model (DOM)
  * Level 2 Specification (Style)
  * http://www.w3.org/TR/DOM-Level-2-Style/
  * Copyright  2000 W3C (MIT, INRIA, Keio), All Rights Reserved.
  *
- * $Id: css_value.h,v 1.17.2.1 2003/05/18 12:34:37 mueller Exp $
  */
 #ifndef _CSS_css_value_h_
 #define _CSS_css_value_h_
@@ -32,6 +31,8 @@
 
 #include <qcolor.h>
 
+#include <kdemacros.h>
+
 namespace DOM {
 
 class CSSStyleDeclarationImpl;
@@ -39,7 +40,7 @@ class CSSRule;
 class CSSValue;
 
 /**
- * The <code> CSSStyleDeclaration </code> interface represents a
+ * The \c CSSStyleDeclaration interface represents a
  * single <a href="http://www.w3.org/TR/REC-CSS2/syndata.html#block">
  * CSS declaration block </a> . This interface may be used to
  * determine the style properties currently set in a block or to set
@@ -47,16 +48,16 @@ class CSSValue;
  *
  *  While an implementation may not recognize all CSS properties
  * within a CSS declaration block, it is expected to provide access to
- * all specified properties through the <code> CSSStyleDeclaration
- * </code> interface. Furthermore, implementations that support a
+ * all specified properties through the \c CSSStyleDeclaration
+ * interface. Furthermore, implementations that support a
  * specific level of CSS should correctly handle <a
  * href="http://www.w3.org/TR/REC-CSS2/about.html#shorthand"> CSS
  * shorthand </a> properties for that level. For a further discussion
- * of shorthand properties, see the <code> CSS2Properties </code>
+ * of shorthand properties, see the \c CSS2Properties
  * interface.
  *
  */
-class CSSStyleDeclaration
+class KHTML_EXPORT CSSStyleDeclaration
 {
 public:
     CSSStyleDeclaration();
@@ -78,7 +79,7 @@ public:
     DOM::DOMString cssText() const;
 
     /**
-     * see @ref cssText
+     * see cssText
      * @exception CSSException
      * SYNTAX_ERR: Raised if the specified CSS string value has a
      * syntax error and is unparsable.
@@ -116,6 +117,7 @@ public:
      * string if the property has not been set.
      *
      */
+    DOM::DOMString getPropertyValue ( const DOM::DOMString &propertyName ) const;
     DOM::DOMString getPropertyValue ( const DOM::DOMString &propertyName );
 
     /**
@@ -124,8 +126,8 @@ public:
      * declaration block. This method returns null if the property is
      * a <a href="http://www.w3.org/TR/REC-CSS2/about.html#shorthand">
      * shorthand </a> property. Shorthand property values can only be
-     * accessed and modified as strings, using the <code>
-     * getPropertyValue </code> and <code> setProperty </code>
+     * accessed and modified as strings, using the
+     * \c getPropertyValue and \c setProperty
      * methods.
      *
      * @param propertyName The name of the CSS property. See the <a
@@ -133,10 +135,11 @@ public:
      * index </a> .
      *
      * @return Returns the value of the property if it has been
-     * explicitly set for this declaration block. Returns the <code>
-     * null </code> if the property has not been set.
+     * explicitly set for this declaration block. Returns the
+     * \c null if the property has not been set.
      *
      */
+    CSSValue getPropertyCSSValue ( const DOM::DOMString &propertyName ) const;
     CSSValue getPropertyCSSValue ( const DOM::DOMString &propertyName );
 
     /**
@@ -161,18 +164,19 @@ public:
 
     /**
      * Used to retrieve the priority of a CSS property (e.g. the
-     * <code> "important" </code> qualifier) if the property has been
+     * \c "important" qualifier) if the property has been
      * explicitly set in this declaration block.
      *
      * @param propertyName The name of the CSS property. See the <a
      * href="http://www.w3.org/TR/REC-CSS2/propidx.html"> CSS property
      * index </a> .
      *
-     * @return A string representing the priority (e.g. <code>
-     * "important" </code> ) if one exists. The empty string if none
+     * @return A string representing the priority (e.g.
+     * \c "important" ) if one exists. The empty string if none
      * exists.
      *
      */
+    DOM::DOMString getPropertyPriority ( const DOM::DOMString &propertyName ) const;
     DOM::DOMString getPropertyPriority ( const DOM::DOMString &propertyName );
 
     /**
@@ -185,8 +189,8 @@ public:
      *
      * @param value The new value of the property.
      *
-     * @param priority The new priority of the property (e.g. <code>
-     * "important" </code> ).
+     * @param priority The new priority of the property (e.g.
+     * \c "important" ).
      *
      * @return
      *
@@ -214,6 +218,7 @@ public:
      * empty string if no property exists at this position.
      *
      */
+    DOM::DOMString item ( unsigned long index ) const;
     DOM::DOMString item ( unsigned long index );
 
     /**
@@ -231,11 +236,11 @@ protected:
 class CSSValueImpl;
 
 /**
- * The <code> CSSValue </code> interface represents a simple or a
+ * The \c CSSValue interface represents a simple or a
  * complexe value.
  *
  */
-class CSSValue
+class KHTML_EXPORT CSSValue
 {
 public:
     CSSValue();
@@ -257,7 +262,8 @@ public:
 	CSS_INHERIT = 0,
         CSS_PRIMITIVE_VALUE = 1,
         CSS_VALUE_LIST = 2,
-        CSS_CUSTOM = 3
+        CSS_CUSTOM = 3,
+        CSS_INITIAL = 4
     };
 
     /**
@@ -267,7 +273,7 @@ public:
     DOM::DOMString cssText() const;
 
     /**
-     * see @ref cssText
+     * see cssText
      * @exception CSSException
      * SYNTAX_ERR: Raised if the specified CSS string value has a
      * syntax error and is unparsable.
@@ -303,11 +309,11 @@ class CSSValueListImpl;
 class CSSValue;
 
 /**
- * The <code> CSSValueList </code> interface provides the absraction
+ * The \c CSSValueList interface provides the absraction
  * of an ordered collection of CSS values.
  *
  */
-class CSSValueList : public CSSValue
+class KHTML_EXPORT CSSValueList : public CSSValue
 {
 public:
     CSSValueList();
@@ -322,9 +328,9 @@ public:
     ~CSSValueList();
 
     /**
-     * The number of <code> CSSValue </code> s in the list. The range
-     * of valid values indices is <code> 0 </code> to <code> length-1
-     * </code> inclusive.
+     * The number of \c CSSValue s in the list. The range
+     * of valid values indices is \c 0 to \c length-1
+     * inclusive.
      *
      */
     unsigned long length() const;
@@ -336,8 +342,8 @@ public:
      *
      * @param index Index into the collection.
      *
-     * @return The style rule at the <code> index </code> position in
-     * the <code> CSSValueList </code> , or <code> null </code> if
+     * @return The style rule at the \c index position in
+     * the \c CSSValueList , or \c null if
      * that is not valid index.
      *
      */
@@ -354,17 +360,17 @@ class RGBColor;
 class Rect;
 
 /**
- * The <code> CSSPrimitiveValue </code> interface represents a single
+ * The \c CSSPrimitiveValue interface represents a single
  * <a href="http://www.w3.org/TR/REC-CSS2/syndata.html#values"> CSS
  * value </a> . This interface may be used to determine the value of a
  * specific style property currently set in a block or to set a
  * specific style properties explicitly within the block. An instance
- * of this interface can be obtained from the <code>
- * getPropertyCSSValue </code> method of the <code>
- * CSSStyleDeclaration </code> interface.
+ * of this interface can be obtained from the
+ * \c getPropertyCSSValue method of the
+ * \c CSSStyleDeclaration interface.
  *
  */
-class CSSPrimitiveValue : public CSSValue
+class KHTML_EXPORT CSSPrimitiveValue : public CSSValue
 {
 public:
     CSSPrimitiveValue();
@@ -422,18 +428,18 @@ public:
      * A method to set the float value with a specified unit. If the
      * property attached with this value can not accept the specified
      * unit or the float value, the value will be unchanged and a
-     * <code> DOMException </code> will be raised.
+     * \c DOMException will be raised.
      *
      * @param unitType A unit code as defined above. The unit code can
-     * only be a float unit type (e.g. <code> NUMBER </code> , <code>
-     * PERCENTAGE </code> , <code> CSS_EMS </code> , <code> CSS_EXS
-     * </code> , <code> CSS_PX </code> , <code> CSS_PX </code> ,
-     * <code> CSS_CM </code> , <code> CSS_MM </code> , <code> CSS_IN
-     * </code> , <code> CSS_PT </code> , <code> CSS_PC </code> ,
-     * <code> CSS_DEG </code> , <code> CSS_RAD </code> , <code>
-     * CSS_GRAD </code> , <code> CSS_MS </code> , <code> CSS_S </code>
-     * , <code> CSS_HZ </code> , <code> CSS_KHZ </code> , <code>
-     * CSS_DIMENSION </code> ).
+     * only be a float unit type (e.g. \c NUMBER ,
+     * \c PERCENTAGE , \c CSS_EMS , \c CSS_EXS
+     * , \c CSS_PX , \c CSS_PX ,
+     * \c CSS_CM , \c CSS_MM , \c CSS_IN
+     * , \c CSS_PT , \c CSS_PC ,
+     * \c CSS_DEG , \c CSS_RAD ,
+     * \c CSS_GRAD , \c CSS_MS , \c CSS_S
+     * , \c CSS_HZ , \c CSS_KHZ ,
+     * \c CSS_DIMENSION  ).
      *
      * @param floatValue The new float value.
      *
@@ -451,19 +457,19 @@ public:
     /**
      * This method is used to get a float value in a specified unit.
      * If this CSS value doesn't contain a float value or can't be
-     * converted into the specified unit, a <code> DOMException
-     * </code> is raised.
+     * converted into the specified unit, a \c DOMException
+     * is raised.
      *
      * @param unitType A unit code to get the float value. The unit
-     * code can only be a float unit type (e.g. <code> CSS_NUMBER
-     * </code> , <code> CSS_PERCENTAGE </code> , <code> CSS_EMS
-     * </code> , <code> CSS_EXS </code> , <code> CSS_PX </code> ,
-     * <code> CSS_PX </code> , <code> CSS_CM </code> , <code> CSS_MM
-     * </code> , <code> CSS_IN </code> , <code> CSS_PT </code> ,
-     * <code> CSS_PC </code> , <code> CSS_DEG </code> , <code> CSS_RAD
-     * </code> , <code> CSS_GRAD </code> , <code> CSS_MS </code> ,
-     * <code> CSS_S </code> , <code> CSS_HZ </code> , <code> CSS_KHZ
-     * </code> , <code> CSS_DIMENSION </code> ).
+     * code can only be a float unit type (e.g. \c CSS_NUMBER
+     * , \c CSS_PERCENTAGE , \c CSS_EMS
+     * , \c CSS_EXS , \c CSS_PX ,
+     * \c CSS_PX , \c CSS_CM , \c CSS_MM
+     * , \c CSS_IN , \c CSS_PT ,
+     * \c CSS_PC , \c CSS_DEG , \c CSS_RAD
+     * , \c CSS_GRAD , \c CSS_MS ,
+     * \c CSS_S , \c CSS_HZ , \c CSS_KHZ
+     * , \c CSS_DIMENSION ).
      *
      * @return The float value in the specified unit.
      *
@@ -473,22 +479,23 @@ public:
      * specified unit.
      *
      */
+    // ### KDE 4: make this const!
     float getFloatValue ( unsigned short unitType );
 
     /**
      * A method to set the string value with a specified unit. If the
      * property attached to this value can't accept the specified unit
-     * or the string value, the value will be unchanged and a <code>
-     * DOMException </code> will be raised.
+     * or the string value, the value will be unchanged and a
+     * \c DOMException will be raised.
      *
      * @param stringType A string code as defined above. The string
-     * code can only be a string unit type (e.g. <code> CSS_URI
-     * </code> , <code> CSS_IDENT </code> , <code> CSS_INHERIT </code>
-     * and <code> CSS_ATTR </code> ).
+     * code can only be a string unit type (e.g. \c CSS_URI
+     * , \c CSS_IDENT , \c CSS_INHERIT
+     * and \c CSS_ATTR ).
      *
-     * @param stringValue The new string value. If the <code>
-     * stringType </code> is equal to <code> CSS_INHERIT </code> , the
-     * <code> stringValue </code> should be <code> inherit </code> .
+     * @param stringValue The new string value. If the
+     * \c stringType is equal to \c CSS_INHERIT , the
+     * \c stringValue should be \c inherit .
      *
      * @return
      * @exception DOMException
@@ -504,26 +511,27 @@ public:
 
     /**
      * This method is used to get the string value in a specified
-     * unit. If the CSS value doesn't contain a string value, a <code>
-     * DOMException </code> is raised.
+     * unit. If the CSS value doesn't contain a string value, a
+     * \c DOMException is raised.
      *
      * @return The string value in the current unit. The current
-     * <code> valueType </code> can only be a string unit type (e.g.
-     * <code> CSS_URI </code> , <code> CSS_IDENT </code> and <code>
-     * CSS_ATTR </code> ).
+     * \c valueType can only be a string unit type (e.g.
+     * \c CSS_URI , \c CSS_IDENT and
+     * \c CSS_ATTR ).
      *
      * @exception DOMException
      * INVALID_ACCESS_ERR: Raises if the CSS value doesn't contain a
      * string value.
      *
      */
+    // ### KDE4: make this const!
     DOM::DOMString getStringValue (  );
 
     /**
      * This method is used to get the Counter value. If this CSS value
-     * doesn't contain a counter value, a <code> DOMException </code>
+     * doesn't contain a counter value, a \c DOMException
      * is raised. Modification to the corresponding style property can
-     * be achieved using the <code> Counter </code> interface.
+     * be achieved using the \c Counter interface.
      *
      * @return The Counter value.
      *
@@ -532,13 +540,14 @@ public:
      * Counter value.
      *
      */
+    // ### KDE4: make this const!
     Counter getCounterValue (  );
 
     /**
      * This method is used to get the Rect value. If this CSS value
-     * doesn't contain a rect value, a <code> DOMException </code> is
+     * doesn't contain a rect value, a \c DOMException is
      * raised. Modification to the corresponding style property can be
-     * achieved using the <code> Rect </code> interface.
+     * achieved using the \c Rect interface.
      *
      * @return The Rect value.
      *
@@ -547,13 +556,14 @@ public:
      * Rect value.
      *
      */
+    // ### KDE4: make this const!
     Rect getRectValue (  );
 
     /**
      * This method is used to get the RGB color. If this CSS value
-     * doesn't contain a RGB color value, a <code> DOMException
-     * </code> is raised. Modification to the corresponding style
-     * property can be achieved using the <code> RGBColor </code>
+     * doesn't contain a RGB color value, a \c DOMException
+     * is raised. Modification to the corresponding style
+     * property can be achieved using the \c RGBColor
      * interface.
      *
      * @return the RGB color value.
@@ -563,20 +573,21 @@ public:
      * return a RGB color value.
      *
      */
+    // ### KDE4: make this const!
     RGBColor getRGBColorValue (  );
 };
 
 
 
 /**
- * The <code> RGBColor </code> interface is used to represent any <a
+ * The \c RGBColor interface is used to represent any <a
  * href="http://www.w3.org/TR/REC-CSS2/syndata.html#value-def-color">
  * RGB color </a> value. This interface reflects the values in the
  * underlying style property. Hence, modifications made through this
  * interface modify the style property.
  *
  */
-class RGBColor
+class KHTML_EXPORT RGBColor
 {
 public:
     RGBColor();
@@ -620,14 +631,14 @@ protected:
 class RectImpl;
 
 /**
- * The <code> Rect </code> interface is used to represent any <a
+ * The \c Rect interface is used to represent any <a
  * href="http://www.w3.org/TR/REC-CSS2/visufx.html#value-def-shape">
  * rect </a> value. This interface reflects the values in the
  * underlying style property. Hence, modifications made through this
  * interface modify the style property.
  *
  */
-class Rect
+class KHTML_EXPORT Rect
 {
     friend class CSSPrimitiveValue;
 public:
@@ -677,14 +688,14 @@ protected:
 class CounterImpl;
 
 /**
- * The <code> Counter </code> interface is used to represent any <a
+ * The \c Counter interface is used to represent any <a
  * href="http://www.w3.org/TR/REC-CSS2/syndata.html#value-def-counter">
  * counter or counters function </a> value. This interface reflects
  * the values in the underlying style property. Hence, modifications
  * made through this interface modify the style property.
  *
  */
-class Counter
+class KHTML_EXPORT Counter
 {
     friend class CSSPrimitiveValue;
 public:
@@ -729,4 +740,5 @@ protected:
 
 } // namespace
 
+
 #endif
