@@ -18,18 +18,21 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
-#ifndef KHTML_H
-#define KHTML_H
+#ifndef KHTMLVIEW_H
+#define KHTMLVIEW_H
 
 // qt includes and classes
 #include <qscrollview.h>
 
+#include <kdelibs_export.h>
+
 class QPainter;
 class QRect;
+template< typename T > class QValueVector;
 
 namespace DOM {
     class HTMLDocumentImpl;
@@ -60,7 +63,7 @@ namespace khtml {
     class RenderWidget;
     class CSSStyleSelector;
     class LineEditWidget;
-    class InlineBox;
+    class CaretBox;
     void applyRule(DOM::CSSProperty *prop);
 }
 
@@ -72,7 +75,7 @@ class KHTMLViewPrivate;
  *
  * Suitable for use as an application's main view.
  **/
-class KHTMLView : public QScrollView
+class KHTML_EXPORT KHTMLView : public QScrollView
 {
     Q_OBJECT
 
@@ -160,10 +163,20 @@ public:
      * ensure the display is up to date
      */
     void layout(); // KDE 4.0: make private
+    /**
+     * Display all accesskeys in small tooltips
+     */
+    void displayAccessKeys();
+
+
 
 signals:
+    void finishedLayout();
     void cleared();
     void zoomView( int );
+    void hideAccessKeys();
+    void repaintAccessKeys();
+    void findAheadActive( bool );
 
 protected:
     void clear();
@@ -193,23 +206,33 @@ protected:
     void keyReleaseEvent ( QKeyEvent *_ke );
     void contentsContextMenuEvent ( QContextMenuEvent *_ce );
     void doAutoScroll();
-
     void timerEvent ( QTimerEvent * );
-
 protected slots:
     void slotPaletteChanged();
     void slotScrollBarMoved();
 
 private slots:
     void tripleClickTimeout();
+    void findTimeout();
+    void accessKeysTimeout();
+
+    /**
+    * @internal
+    * @since 3.4
+    * used for autoscrolling with MMB
+    */
+    void slotMouseScrollTimer();
 
 private:
+
     void scheduleRelayout(khtml::RenderObject* clippedObj=0);
     void unscheduleRelayout();
 
-    void scheduleRepaint(int x, int y, int w, int h);
+    void scheduleRepaint(int x, int y, int w, int h, bool asap=false);
     void unscheduleRepaint();
 
+    bool needsFullRepaint() const;
+
     void closeChildDialogs();
     bool dialogsAllowed();
 
@@ -223,6 +246,14 @@ private:
      **/
     void paint(QPainter *p, const QRect &rc, int yOff = 0, bool *more = 0);
 
+#if 0
+    /**
+     * Paints the HTML document to a QPainter.
+     * The document will be scaled to match the width.
+     **/
+    void paint(QPainter *p, int width);
+#endif
+
     /**
      * Get/set the CSS Media Type.
      *
@@ -236,9 +267,15 @@ private:
     void setMediaType( const QString &medium );
     QString mediaType() const;
 
+    bool pagedMode() const;
+
     bool scrollTo(const QRect &);
 
-    void focusNextPrevNode(bool next);
+    bool focusNextPrevNode(bool next);
+    bool handleAccessKey(const QKeyEvent* ev);
+    bool focusNodeWithAccessKey(QChar c, KHTMLView* caller = NULL);
+    QMap< DOM::ElementImpl*, QChar > buildFallbackAccessKeys() const;
+    void displayAccessKeys( KHTMLView* caller, KHTMLView* origview, QValueVector< QChar >& taken, bool use_fallbacks );
 
     void useSlowRepaints();
 
@@ -247,6 +284,7 @@ private:
     void init();
 
     DOM::NodeImpl *nodeUnderMouse() const;
+    DOM::NodeImpl *nonSharedNodeUnderMouse() const;
 
     void restoreScrollBar();
 
@@ -257,13 +295,20 @@ private:
     void addNonPasswordStorableSite( const QString& host );
     bool nonPasswordStorableSite( const QString& host ) const;
 
-    bool dispatchMouseEvent(int eventId, DOM::NodeImpl *targetNode, bool cancelable,
+    bool dispatchMouseEvent(int eventId, DOM::NodeImpl *targetNode,
+			    DOM::NodeImpl *targetNodeNonShared, bool cancelable,
 			    int detail,QMouseEvent *_mouse, bool setUnder,
 			    int mouseEventType);
     bool dispatchKeyEvent( QKeyEvent *_ke );
     bool dispatchKeyEventHelper( QKeyEvent *_ke, bool generate_keypress );
 
-    void complete();
+    void complete( bool pendingAction );
+
+#ifndef KHTML_NO_TYPE_AHEAD_FIND
+    void findAhead(bool increase);
+    void updateFindAheadTimeout();
+    void startFindAhead( bool linksOnly );
+#endif // KHTML_NO_TYPE_AHEAD_FIND
 
 #ifndef KHTML_NO_CARET
     // -- caret-related member functions (for caretMode as well as designMode)
@@ -293,10 +338,10 @@ private:
     /** inquires the current caret position and stores it in the caret view
      * context. Also resets the blink frequency timer. It will not display
      * the caret on the canvas.
-     * @param hintBox inline box whose coordinates will be used if the
+     * @param hintBox caret box whose coordinates will be used if the
      * caret position could not be determined otherwise.
      */
-    void recalcAndStoreCaretPos(khtml::InlineBox *hintBox = 0);
+    void recalcAndStoreCaretPos(khtml::CaretBox *hintBox = 0);
     /** displays the caret and reinitializes the blink frequency timer.
      *
      * The caret will only be displayed on screen if the view has focus, or
@@ -355,11 +400,11 @@ private:
      * The caret is switched off, the position recalculated with respect to
      * the new position. The caret will only be redisplayed if it is on an
      * editable node, in design mode, or in caret mode.
-     * @param hintBox inline box whose coordinates will be used if the
+     * @param hintBox caret box whose coordinates will be used if the
      * caret position could not be determined otherwise.
      * @return @p true if the caret has been displayed.
      */
-    bool placeCaret(khtml::InlineBox *hintBox = 0);
+    bool placeCaret(khtml::CaretBox *hintBox = 0);
 
     /** extends the selection up to the current caret position.
      *
@@ -417,7 +462,7 @@ private:
 
     /** moves the caret to the given position and displays it.
      *
-     * If the node is an invalid place, the function sets the caret to an
+     * If the node resembles an invalid position, the function sets the caret to a
      * nearby node that is valid.
      *
      * @param node node to be set to
@@ -448,7 +493,6 @@ private:
     void moveCaretBy(bool next, CaretMovement cmv, int n);
 
     /** moves the caret by line.
-     * @internal only meant to be called from within moveCaretBy
      */
     void moveCaretByLine(bool next, int n);
 
@@ -469,10 +513,10 @@ private:
      *
      * This method is only suitable if the new caret position has already been
      * determined.
-     * @param hintBox inline box whose coordinates will be used if the
+     * @param hintBox caret box whose coordinates will be used if the
      * caret position could not be determined otherwise.
      */
-    void placeCaretOnChar(khtml::InlineBox *hintBox);
+    void placeCaretOnChar(khtml::CaretBox *hintBox);
 
     /** does the actual caret placement so that it becomes visible at
      * the new position.
@@ -480,7 +524,7 @@ private:
      * Additionally, it calculates the new caret position from the given
      * box and coordinates.
      *
-     * @param caretBox inline box serving as a measurement point for offset.
+     * @param caretBox caret box serving as a measurement point for offset.
      * @param x x-coordinate relative to containing block. The offset will
      *		be approximated as closely as possible to this coordinate,
      *		but never less than caretBox->xPos() and greater than
@@ -490,7 +534,7 @@ private:
      * @param absy absolute y-coordinate of containing block, needed for
      *		calculation of final caret position
      */
-    void placeCaretOnLine(khtml::InlineBox *caretBox, int x, int absx, int absy);
+    void placeCaretOnLine(khtml::CaretBox *caretBox, int x, int absx, int absy);
 
     /** moves the caret by a page length.
      * @param next @p true, move down, @p false, move up.
