@@ -15,16 +15,13 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef __kio_jobclasses_h__
 #define __kio_jobclasses_h__
 
-#include <kurl.h>
-#include <kdemacros.h>
-
 #include <qobject.h>
 #include <qptrlist.h>
 #include <qstring.h>
@@ -34,11 +31,14 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include <kurl.h>
 #include <kio/global.h>
 
 class Observer;
 class QTimer;
 
+#define KIO_COPYJOB_HAS_SETINTERACTIVE // new in 3.4. Used by kio_trash.
+
 namespace KIO {
 
     class Slave;
@@ -65,7 +65,7 @@ namespace KIO {
      * @see KIO::Scheduler
      * @see KIO::Slave
      */
-    class Job : public QObject {
+    class KIO_EXPORT Job : public QObject {
         Q_OBJECT
 
     protected:
@@ -103,7 +103,7 @@ namespace KIO {
         /**
 	 * Returns the error text if there has been an error.
          * Only call if error is not 0.
-         * This is really internal, better use errorString or errorDialog.
+         * This is really internal, better use errorString() or errorDialog().
 	 *
          * @return a string to help understand the error, usually the url
          * related to the error. Only valid if error() is not 0.
@@ -160,23 +160,60 @@ namespace KIO {
          *
          * The default is false.
          *
-         * See also isAutoErrorHandlingEnabled , showErrorDialog
-         *
          * @param enable enable or disable automatic error handling
          * @param parentWidget the parent widget, passed to showErrorDialog.
 	 *        Can be 0 for top-level
-	 * @see isAutoErrorHandlingEnabled()
+	 * @see isAutoErrorHandlingEnabled(), showErrorDialog()
          */
         void setAutoErrorHandlingEnabled( bool enable, QWidget *parentWidget = 0 );
 
         /**
          * Returns whether automatic error handling is enabled or disabled.
-         * See also setAutoErrorHandlingEnabled .
 	 * @return true if automatic error handling is enabled
 	 * @see setAutoErrorHandlingEnabled()
          */
         bool isAutoErrorHandlingEnabled() const;
 
+        /**
+         * Enable or disable the automatic warning handling. When automatic
+         * warning handling is enabled and an error occurs, then a message box
+         * is displayed with the warning message
+         *
+         * The default is true.
+         *
+         * See also isAutoWarningHandlingEnabled , showErrorDialog
+         *
+         * @param enable enable or disable automatic warning handling
+         * @see isAutoWarningHandlingEnabled()
+         * @since 3.5
+         */
+        void setAutoWarningHandlingEnabled( bool enable );
+
+        /**
+         * Returns whether automatic warning handling is enabled or disabled.
+         * See also setAutoWarningHandlingEnabled .
+         * @return true if automatic warning handling is enabled
+         * @see setAutoWarningHandlingEnabled()
+         * @since 3.5
+         */
+        bool isAutoWarningHandlingEnabled() const;
+
+        /**
+         * Enable or disable the message display from the job.
+         *
+         * The default is true.
+         * @param enable enable or disable message display
+         * @since 3.4.1
+         */
+        void setInteractive(bool enable);
+
+        /**
+         * Returns whether message display is enabled or disabled.
+         * @return true if message display is enabled
+         * @see setInteractive()
+         * @since 3.4.1
+         */
+        bool isInteractive() const;
         /**
          * Associate this job with a window given by @p window.
 	 * @param window the window to associate to
@@ -299,7 +336,16 @@ namespace KIO {
 	 * @param msg the info message
          */
         void infoMessage( KIO::Job *job, const QString & msg );
-        // KDE 3.0: Separate rich-text string from plain-text string, for different widgets.
+        // KDE4: Separate rich-text string from plain-text string, for different widgets.
+
+        /**
+         * Emitted to display a warning about this job, as sent by the slave.
+         * @param job the job that emitted this signal
+         * @param msg the info message
+         * @since 3.5
+         */
+        void warning( KIO::Job *job, const QString & msg );
+        // KDE4: Separate rich-text string from plain-text string, for different widgets.
 
         /**
          * Emitted when the slave successfully connected to the host.
@@ -338,9 +384,9 @@ namespace KIO {
         /**
          * Emitted to display information about the speed of this job.
 	 * @param job the job that emitted this signal
-	 * @param bytes_per_second the speed in bytes/s
+	 * @param speed the speed in bytes/s
          */
-        void speed( KIO::Job *job, unsigned long bytes_per_second );
+        void speed( KIO::Job *job, unsigned long speed );
 
     protected slots:
         /**
@@ -356,10 +402,10 @@ namespace KIO {
         /**
          * Forward signal from subjob.
 	 * @param job the subjob
-	 * @param bytes_per_second the speed in bytes/s
+	 * @param speed the speed in bytes/s
 	 * @see speed()
          */
-        void slotSpeed( KIO::Job *job, unsigned long bytes_per_second );
+        void slotSpeed( KIO::Job *job, unsigned long speed );
         /**
          * Forward signal from subjob.
 	 * @param job the subjob
@@ -394,6 +440,15 @@ namespace KIO {
 	 * @param job the subjob to add
          */
         virtual void removeSubjob( Job *job );
+        /**
+         * Overloaded version of removeSubjob
+         * @param job the subjob to remove
+         * @param mergeMetaData if set, the metadata received by the subjob is
+         *                      merged into this job.
+         * @param emitResultIfLast if this was the last subjob, emit result,
+         *                         i.e. terminate this job.
+         */
+        void removeSubjob( Job *job, bool mergeMetaData, bool emitResultIfLast ); // KDE4: merge with above, with =true to both
 
         /**
          * Utility function for inherited jobs.
@@ -409,9 +464,9 @@ namespace KIO {
          * Utility function for inherited jobs.
          * Emits the speed signal and starts the timer for removing that info
 	 *
-	 * @param bytes_per_second the speed in bytes_per_second
+	 * @param speed the speed in bytes/s
          */
-        void emitSpeed( unsigned long bytes_per_second );
+        void emitSpeed( unsigned long speed );
 
         /**
          * Utility function to emit the result signal, and suicide this job.
@@ -458,7 +513,7 @@ namespace KIO {
      * Other jobs are high-level jobs (CopyJob, DeleteJob, FileCopyJob...)
      * that manage subjobs but aren't scheduled directly.
      */
-    class SimpleJob : public KIO::Job {
+    class KIO_EXPORT SimpleJob : public KIO::Job {
     Q_OBJECT
 
     public:
@@ -544,14 +599,14 @@ namespace KIO {
          * @internal
          * Called on a slave's warning.
          */
-        void slotWarning( const QString & );
+        void slotWarning( const QString & ); // KDE4: make virtual
 
         /**
          * Called on a slave's info message.
 	 * @param s the info message
 	 * @see infoMessage()
          */
-        void slotInfoMessage( const QString &s );
+        void slotInfoMessage( const QString &s ); // KDE4: make virtual
 
         /**
          * Called on a slave's connected signal.
@@ -567,10 +622,10 @@ namespace KIO {
         void slotProcessedSize( KIO::filesize_t data_size );
         /**
          * Forward signal from the slave.
-	 * @param bytes_per_second the speed in bytes/s
+	 * @param speed the speed in bytes/s
 	 * @see speed()
          */
-        void slotSpeed( unsigned long bytes_per_second );
+        void slotSpeed( unsigned long speed );
 
         /**
          * MetaData from the slave is received.
@@ -618,7 +673,7 @@ namespace KIO {
      * A KIO job that retrieves information about a file or directory.
      * @see KIO::stat()
      */
-    class StatJob : public SimpleJob {
+    class KIO_EXPORT StatJob : public SimpleJob {
 
     Q_OBJECT
 
@@ -705,13 +760,107 @@ namespace KIO {
         StatJobPrivate *d;
     };
 
+    /**
+     * A KIO job that creates a directory
+     * @see KIO::mkdir()
+     * @since 3.3
+     */
+    class KIO_EXPORT MkdirJob : public SimpleJob {
+
+    Q_OBJECT
+
+    public:
+        /**
+	 * Do not use this constructor to create a MkdirJob, use KIO::mkdir() instead.
+	 * @param url the url of the file or directory to check
+	 * @param command the command to issue
+	 * @param packedArgs the arguments
+	 * @param showProgressInfo true to show progress information to the user
+	 */
+        MkdirJob(const KURL& url, int command, const QByteArray &packedArgs, bool showProgressInfo);
+
+        /**
+	 * @internal
+         * Called by the scheduler when a @p slave gets to
+         * work on this job.
+	 * @param slave the slave that starts working on this job
+         */
+        virtual void start( Slave *slave );
+
+    signals:
+        /**
+         * Signals a redirection.
+         * Use to update the URL shown to the user.
+         * The redirection itself is handled internally.
+	 * @param job the job that is redirected
+	 * @param url the new url
+         */
+        void redirection( KIO::Job *job, const KURL &url );
+
+        /**
+         * Signals a permanent redirection.
+         * The redirection itself is handled internally.
+	 * @param job the job that is redirected
+	 * @param fromUrl the original URL
+	 * @param toUrl the new URL
+         */
+        void permanentRedirection( KIO::Job *job, const KURL &fromUrl, const KURL &toUrl );
+
+    protected slots:
+        void slotRedirection( const KURL &url);
+        virtual void slotFinished();
+
+    protected:
+        KURL m_redirectionURL;
+
+    protected:
+	virtual void virtual_hook( int id, void* data );
+    private:
+        class MkdirJobPrivate;
+        MkdirJobPrivate *d;
+    };
+
+    /**
+     * @internal
+     * Used for direct copy from or to the local filesystem (i.e. SlaveBase::copy())
+     */
+    class KIO_EXPORT DirectCopyJob : public SimpleJob {
+    Q_OBJECT
+
+    public:
+        /**
+         * Do not create a DirectCopyJob. Use KIO::copy() or KIO::file_copy() instead.
+         */
+        DirectCopyJob(const KURL& url, int command, const QByteArray &packedArgs,
+                      bool showProgressInfo);
+        /**
+	 * @internal
+         * Called by the scheduler when a @p slave gets to
+         * work on this job.
+	 * @param slave the slave that starts working on this job
+         */
+        virtual void start(Slave *slave);
+
+    signals:
+        /**
+         * @internal
+         * Emitted if the job found an existing partial file
+         * and supports resuming. Used by FileCopyJob.
+         */
+        void canResume( KIO::Job *job, KIO::filesize_t offset );
+
+    private slots:
+        void slotCanResume( KIO::filesize_t offset );
+    };
+
+
     /**
      * The transfer job pumps data into and/or out of a Slave.
      * Data is sent to the slave on request of the slave ( dataReq).
      * If data coming from the slave can not be handled, the
      * reading of data from the slave should be suspended.
      */
-    class TransferJob : public SimpleJob {
+    class KIO_EXPORT TransferJob : public SimpleJob {
     Q_OBJECT
 
     public:
@@ -891,13 +1040,72 @@ namespace KIO {
 	class TransferJobPrivate *d;
     };
 
+    /**
+     * StoredTransferJob is a TransferJob (for downloading or uploading data) that
+     * also stores a QByteArray with the data, making it simpler to use than the
+     * standard TransferJob.
+     *
+     * For KIO::storedGet it puts the data into the member QByteArray, so the user
+     * of this class can get hold of the whole data at once by calling data()
+     * when the result signal is emitted.
+     * You should only use StoredTransferJob to download data if you cannot
+     * process the data by chunks while it's being downloaded, since storing
+     * everything in a QByteArray can potentially require a lot of memory.
+     *
+     * For KIO::storedPut the user of this class simply provides the bytearray from
+     * the start, and the job takes care of uploading it.
+     * You should only use StoredTransferJob to upload data if you cannot
+     * provide the in chunks while it's being uploaded, since storing
+     * everything in a QByteArray can potentially require a lot of memory.
+     *
+     * @since 3.3
+     */
+    class KIO_EXPORT StoredTransferJob : public KIO::TransferJob {
+        Q_OBJECT
+
+    public:
+       /**
+	* Do not create a StoredTransferJob. Use storedGet() or storedPut()
+	* instead.
+	* @param url the url to get or put
+	* @param command the command to issue
+	* @param packedArgs the arguments
+	* @param _staticData additional data to transmit (e.g. in a HTTP Post)
+	* @param showProgressInfo true to show progress information to the user
+	*/
+        StoredTransferJob(const KURL& url, int command,
+                          const QByteArray &packedArgs,
+                          const QByteArray &_staticData,
+                          bool showProgressInfo);
+
+        /**
+         * Set data to be uploaded. This is for put jobs.
+         * Automatically called by KIO::storedPut(const QByteArray &, ...),
+         * do not call this yourself.
+         */
+        void setData( const QByteArray& arr );
+
+        /**
+         * Get hold of the downloaded data. This is for get jobs.
+         * You're supposed to call this only from the slot connected to the result() signal.
+         */
+        QByteArray data() const { return m_data; }
+
+    private slots:
+        void slotStoredData( KIO::Job *job, const QByteArray &data );
+        void slotStoredDataReq( KIO::Job *job, QByteArray &data );
+    private:
+        QByteArray m_data;
+        int m_uploadOffset;
+    };
+
     /**
      * The MultiGetJob is a TransferJob that allows you to get
      * several files from a single server. Don't create directly,
      * but use KIO::multi_get() instead.
      * @see KIO::multi_get()
      */
-    class MultiGetJob : public TransferJob {
+    class KIO_EXPORT MultiGetJob : public TransferJob {
     Q_OBJECT
 
     public:
@@ -985,7 +1193,7 @@ namespace KIO {
      * but use KIO::mimetype() instead.
      * @see KIO::mimetype()
      */
-    class MimetypeJob : public TransferJob {
+    class KIO_EXPORT MimetypeJob : public TransferJob {
     Q_OBJECT
 
     public:
@@ -1027,7 +1235,7 @@ namespace KIO {
      * @see KIO::file_copy()
      * @see KIO::file_move()
      */
-    class FileCopyJob : public Job {
+    class KIO_EXPORT FileCopyJob : public Job {
     Q_OBJECT
 
     public:
@@ -1111,9 +1319,14 @@ namespace KIO {
     protected:
         void startCopyJob();
         void startCopyJob(const KURL &slave_url);
+        void startRenameJob(const KURL &slave_url);
         void startDataPump();
         void connectSubjob( SimpleJob * job );
 
+    private:
+        void startBestCopyMethod();
+
+    protected:
         KURL m_src;
         KURL m_dest;
         int m_permissions;
@@ -1142,7 +1355,7 @@ namespace KIO {
      * @see KIO::listRecursive()
      * @see KIO::listDir()
      */
-    class ListJob : public SimpleJob {
+    class KIO_EXPORT ListJob : public SimpleJob {
     Q_OBJECT
 
     public:
@@ -1168,6 +1381,14 @@ namespace KIO {
          */
         virtual void start( Slave *slave );
 
+        /**
+         * Returns the ListJob's redirection URL. This will be invalid if there
+         * was no redirection.
+         * @return the redirection url
+         * @since 3.4.1
+         */
+        const KURL& redirectionURL() const { return m_redirectionURL; }
+
         /**
          * Do not apply any KIOSK restrictions to this job.
          * @since 3.2
@@ -1204,7 +1425,7 @@ namespace KIO {
 	 * @since 3.1
          */
         void permanentRedirection( KIO::Job *job, const KURL &fromUrl, const KURL &toUrl );
-        
+
     protected slots:
         virtual void slotFinished( );
         virtual void slotMetaData( const KIO::MetaData &_metaData);
@@ -1225,7 +1446,8 @@ namespace KIO {
 	class ListJobPrivate* d;
     };
 
-    struct CopyInfo
+    /// @internal
+    struct KIO_EXPORT CopyInfo
     {
         KURL uSource;
         KURL uDest;
@@ -1249,7 +1471,7 @@ namespace KIO {
      * @see KIO::link()
      * @see KIO::linkAs()
      */
-    class CopyJob : public Job {
+    class KIO_EXPORT CopyJob : public Job {
     Q_OBJECT
 
     public:
@@ -1277,6 +1499,8 @@ namespace KIO {
 	 */
         CopyJob( const KURL::List& src, const KURL& dest, CopyMode mode, bool asMethod, bool showProgressInfo );
 
+        virtual ~CopyJob();
+
 	/**
 	 * Returns the list of source URLs.
 	 * @return the list of source URLs.
@@ -1289,6 +1513,31 @@ namespace KIO {
 	 */
         KURL destURL() const { return m_dest; }
 
+        /**
+         * By default the permissions of the copied files will be those of the source files.
+         *
+         * But when copying "template" files to "new" files, people prefer the umask
+         * to apply, rather than the template's permissions.
+         * For that case, call setDefaultPermissions(true)
+         *
+         * TODO KDE4: consider adding this as bool to copy/copyAs?
+         * @since 3.2.3
+         */
+        void setDefaultPermissions( bool b );
+
+        /**
+         * When an error happens while copying/moving a file, the user will be presented with
+         * a dialog for skipping the file that can't be copied/moved.
+         * Or if the error is that the destination file already exists, the standard
+         * rename dialog is shown.
+         * If the program doesn't want CopyJob to show dialogs, but to simply fail on error,
+         * call setInteractive( false ).
+         *
+         * KDE4: remove, already in Job
+         * @since 3.4
+         */
+        void setInteractive( bool b );
+
     signals:
 
         /**
@@ -1389,6 +1638,7 @@ namespace KIO {
         void copyingLinkDone( KIO::Job *job, const KURL &from, const QString& target, const KURL& to );
 
     protected:
+        void statCurrentSrc();
         void statNextSrc();
 
         // Those aren't slots but submethods for slotResult.
@@ -1403,6 +1653,12 @@ namespace KIO {
         void slotResultDeletingDirs( KIO::Job * job );
         void deleteNextDir();
         void skip( const KURL & sourceURL );
+        void slotResultRenaming( KIO::Job * job );
+    private:
+        void startRenameJob(const KURL &slave_url);
+        bool shouldOverwrite( const QString& path ) const;
+        bool shouldSkip( const QString& path ) const;
+        void skipSrc();
 
     protected slots:
         void slotStart();
@@ -1422,7 +1678,8 @@ namespace KIO {
     private:
         CopyMode m_mode;
         bool m_asMethod;
-        enum { DEST_NOT_STATED, DEST_IS_DIR, DEST_IS_FILE, DEST_DOESNT_EXIST } destinationState;
+        enum DestinationState { DEST_NOT_STATED, DEST_IS_DIR, DEST_IS_FILE, DEST_DOESNT_EXIST };
+        DestinationState destinationState;
         enum { STATE_STATING, STATE_RENAMING, STATE_LISTING, STATE_CREATING_DIRS,
                STATE_CONFLICT_CREATING_DIRS, STATE_COPYING_FILES, STATE_CONFLICT_COPYING_FILES,
                STATE_DELETING_DIRS } state;
@@ -1456,7 +1713,9 @@ namespace KIO {
     protected:
 	virtual void virtual_hook( int id, void* data );
     private:
-	class CopyJobPrivate* d;
+	class CopyJobPrivate;
+        CopyJobPrivate* d;
+        friend class CopyJobPrivate; // for DestinationState
     };
 
     /**
@@ -1465,7 +1724,7 @@ namespace KIO {
      *
      * @see KIO::del()
      */
-    class DeleteJob : public Job {
+    class KIO_EXPORT DeleteJob : public Job {
     Q_OBJECT
 
     public:
@@ -1554,7 +1813,7 @@ namespace KIO {
         KURL::List m_srcList;
         KURL::List::Iterator m_currentStat;
 	QStringList m_parentDirs;
-        bool m_shred;
+        bool m_shred; // BIC: remove in KDE4
         QTimer *m_reportTimer;
     protected:
         /** \internal */
