@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #ifndef __slavebase_h
@@ -26,6 +26,7 @@
 #include <kio/authinfo.h>
 
 class DCOPClient;
+class KRemoteEncoding;
 
 namespace KIO {
 
@@ -41,7 +42,7 @@ class SlaveBasePrivate;
  *
  * A call to foo() results in a call to slotFoo() on the other end.
  */
-class SlaveBase
+class KIO_EXPORT SlaveBase
 {
 public:
     SlaveBase( const QCString &protocol, const QCString &pool_socket, const QCString &app_socket);
@@ -142,6 +143,7 @@ public:
     /**
      * Call this in listDir, each time you have a bunch of entries
      * to report.
+     * @param _entry The UDSEntry containing all of the object attributes.
      */
     void listEntries( const UDSEntryList& _entry );
 
@@ -211,7 +213,7 @@ public:
     void errorPage();
 
     /**
-     * Call this in mimetype, when you know the mimetype.
+     * Call this in mimetype() and in get(), when you know the mimetype.
      * See mimetype about other ways to implement it.
      */
     void mimeType( const QString &_type );
@@ -230,9 +232,7 @@ public:
     enum MessageBoxType { QuestionYesNo = 1, WarningYesNo = 2, WarningContinueCancel = 3, WarningYesNoCancel = 4, Information = 5, SSLMessageBox = 6 };
 
     /**
-     * Call this to show a message box from the slave (it will in fact be handled
-     * by kio_uiserver, so that the progress info dialog for the slave is hidden
-     * while this message box is shown)
+     * Call this to show a message box from the slave
      * @param type type of message box: QuestionYesNo, WarningYesNo, WarningContinueCancel...
      * @param text Message string. May contain newlines.
      * @param caption Message box title.
@@ -249,6 +249,28 @@ public:
                     const QString &buttonYes = QString::null,
                     const QString &buttonNo = QString::null );
 
+    /**
+     * Call this to show a message box from the slave
+     * @param text Message string. May contain newlines.
+     * @param type type of message box: QuestionYesNo, WarningYesNo, WarningContinueCancel...
+     * @param caption Message box title.
+     * @param buttonYes The text for the first button.
+     *                  The default is i18n("&Yes").
+     * @param buttonNo  The text for the second button.
+     *                  The default is i18n("&No").
+     * Note: for ContinueCancel, buttonYes is the continue button and buttonNo is unused.
+     *       and for Information, none is used.
+     * @param dontAskAgainName A checkbox is added with which further confirmation can be turned off.
+     *        The string is used to lookup and store the setting in kioslaverc.
+     * @return a button code, as defined in KMessageBox, or 0 on communication error.
+     * @since 3.3
+     */
+    int messageBox( const QString &text, MessageBoxType type,
+                    const QString &caption = QString::null,
+                    const QString &buttonYes = QString::null,
+                    const QString &buttonNo = QString::null,
+                    const QString &dontAskAgainName = QString::null );
+
     /**
      * Sets meta-data to be send to the application before the first
      * data() or finished() signal.
@@ -290,6 +312,15 @@ public:
      */
     KConfigBase* config();
 
+    /**
+     * Returns an object that can translate remote filenames into proper
+     * Unicode forms. This encoding can be set by the user.
+     *
+     * @since 3.3
+     */
+    KRemoteEncoding* remoteEncoding();
+
+
     ///////////
     // Commands sent by the job, the slave has to
     // override what it wants to implement
@@ -437,6 +468,9 @@ public:
      */
     virtual void del( const KURL &url, bool isfile);
 
+    // TODO KDE4: add setLinkDest() or something, to modify symlink targets.
+    // Will be used for kio_file but also kio_remote (#97129)
+
     /**
      * Used for any command that is specific to this slave (protocol)
      * Examples are : HTTP POST, mount and unmount (kio_file)
@@ -546,7 +580,8 @@ public:
      * frame exceeded (to make sure the app gets some
      * items in time but not too many items one by one
      * as this will cause a drastic performance penalty)
-     * @param ready set to true after emitting all items. _entry is not
+     * @param _entry The UDSEntry containing all of the object attributes.
+     * @param ready set to true after emitting all items. @p _entry is not
      *        used in this case
      */
     void listEntry( const UDSEntry& _entry, bool ready);
@@ -594,9 +629,14 @@ public:
      * }
      * \endcode
      *
-     * NOTE: A call to this function can fail and return @p false,
+     * \note You should consider using checkCachedAuthentication() to
+     * see if the password is available in kpasswdserver before calling
+     * this function.
+     *
+     * \note A call to this function can fail and return @p false,
      * if the UIServer could not be started for whatever reason.
      *
+     * @see checkCachedAuthentication
      * @param info  See AuthInfo.
      * @param errorMsg Error message to show
      * @return      @p true if user clicks on "OK", @p false otherwsie.
